@@ -22,7 +22,8 @@ class AsyncClient(client.Client):
 
     :param logger: To enable logging set to ``True`` or pass a logger object to
                    use. To disable logging set to ``False``. The default is
-                   ``False``.
+                   ``False``. Note that fatal errors are logged even when
+                   ``logger`` is ``False``.
     :param json: An alternative json module to use for encoding and decoding
                  packets. Custom json modules must have ``dumps`` and ``loads``
                  functions that are compatible with the standard library
@@ -182,9 +183,10 @@ class AsyncClient(client.Client):
             raise exceptions.ConnectionError(
                 'Connection refused by the server')
         if r.status < 200 or r.status >= 300:
+            self._reset()
             raise exceptions.ConnectionError(
                 'Unexpected status code {} in server response'.format(
-                    r.status))
+                    r.status), await r.json())
         try:
             p = payload.Payload(encoded_payload=await r.read())
         except ValueError:
@@ -242,6 +244,17 @@ class AsyncClient(client.Client):
         if self.http is None or self.http.closed:  # pragma: no cover
             self.http = aiohttp.ClientSession()
 
+        # extract any new cookies passed in a header so that they can also be
+        # sent the the WebSocket route
+        cookies = {}
+        for header, value in headers.items():
+            if header.lower() == 'cookie':
+                cookies = dict(
+                    [cookie.split('=', 1) for cookie in value.split('; ')])
+                del headers[header]
+                break
+        self.http.cookie_jar.update_cookies(cookies)
+
         try:
             if not self.ssl_verify:
                 ssl_context = ssl.create_default_context()
@@ -494,12 +507,19 @@ class AsyncClient(client.Client):
                 break
             except Exception as e:
                 self.logger.info(
-                    'Unexpected error "%s", aborting', str(e))
+                    'Unexpected error receiving packet: "%s", aborting',
+                    str(e))
                 await self.queue.put(None)
                 break
             if isinstance(p, six.text_type):  # pragma: no cover
                 p = p.encode('utf-8')
-            pkt = packet.Packet(encoded_packet=p)
+            try:
+                pkt = packet.Packet(encoded_packet=p)
+            except Exception as e:  # pragma: no cover
+                self.logger.info(
+                    'Unexpected error decoding packet: "%s", aborting', str(e))
+                await self.queue.put(None)
+                break
             await self._receive_packet(pkt)
 
         self.logger.info('Waiting for write loop task to end')
